/*
	ldaXML.c
	Par Tuxgasy <tuxgasy@tuxfamily.org>
	
	This file is part of Foobar.
	
	You may redistribute it and/or modify it under the terms of the
	GNU General Public License, as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later version.
	
	Lda is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Lda; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>

#include "ldaXML.h"

int is_file (const char *filename){
	struct stat buf;
	if (stat (filename, &buf) == -1)
		return (0);
	if (S_ISREG (buf.st_mode))
		return (1);
	return (0);
}

xmlDocPtr initListe(char *fichierXML){
	xmlDocPtr ptrFic;
	xmlNodePtr racine;
	
	/* Vérification du fichier passé en argument */
	if(!is_file(fichierXML)){
		fprintf(stderr, "Impossible de trouver le fichier:\n%s\n", fichierXML);
		return NULL;
	}
	
	/* Ouverture du fichier XML */
	ptrFic = xmlParseFile(fichierXML);
	if(ptrFic == NULL){
		fprintf(stderr, "Fichier XML invalide:\n%s\n", fichierXML);
		return NULL;
	}

	/* Lecture */
	racine = xmlDocGetRootElement(ptrFic);
	if(racine == NULL){
		fprintf(stderr, "Fichier XML vide\n");
		return NULL;
	}
	
	return ptrFic;
}

void show (xmlNodePtr node){

	if ((node->type == XML_CDATA_SECTION_NODE) || (node->type == XML_TEXT_NODE)){
		xmlChar *path = xmlGetNodePath (node);
		xmlNodePtr n;
		for (n = node; n; n = n->next){
			if (n->children){
				show (n->children);
			}
		}
		printf ("%s -> '%s'\n", path,
		node->content ? (char *) node->content : "(null)");
		xmlFree (path);
	}
	else if (node->type == XML_ELEMENT_NODE){
		xmlNodePtr n;
		for (n = node; n; n = n->next){
			if (n->children){
				show (n->children);
			}
		}
	}
}

char *getCategoryById(xmlDocPtr ptrFic, int Id){
	xmlXPathContextPtr xmlperms_context = NULL;
	xmlXPathObjectPtr xmlobject;
	const char path_template[] = "/liste_articles/category[%d]";
	xmlChar *path;
	char *ptrCategory;
	
	xmlXPathInit();
	xmlperms_context = xmlXPathNewContext(ptrFic);
	
	path = malloc(sizeof(path_template)+sizeof(int));
	ptrCategory = malloc(sizeof(char)*50);
	
	if(!path || !ptrCategory){
		fprintf(stderr, "Impossible d'allouer de la memoire\n");
		return NULL;
	}
	sprintf((char*)path, path_template, Id);
	//printf("Path : %s\n", path);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	//printf("Type de l'objet: %d\n", xmlobject->type);
	//printf("Nbre de noeuds: %d\n", xmlobject->nodesetval->nodeNr);
	if(xmlobject->nodesetval->nodeNr == 0){
		sprintf(ptrCategory, "%s", "FIN");
		return ptrCategory;
	}
	//printf("Type node: %d\n", xmlobject->nodesetval->nodeTab[0]->type);
	
	ptrCategory = xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar *)"name");
	//printf("Attribut name=%s\n", ptrCategory);
	
	xmlFree(path);
	xmlXPathFreeObject(xmlobject);
	xmlXPathFreeContext(xmlperms_context);
	
	return ptrCategory;
}

char *getSousCategoryByCategory(xmlDocPtr ptrFic, int IdCat, int IdSSCat){
	xmlXPathContextPtr xmlperms_context = NULL;
	xmlXPathObjectPtr xmlobject;
	const char path_template[] = "/liste_articles/category[%d]/sous_category[%d]";
	xmlChar *path;
	char *ptrSSCategory;
	
	xmlXPathInit();
	xmlperms_context = xmlXPathNewContext(ptrFic);
	
	path = malloc(sizeof(path_template)+sizeof(int));
	ptrSSCategory = malloc(sizeof(char)*50);
	
	if(!path || !ptrSSCategory){
		fprintf(stderr, "Impossible d'allouer de la memoire\n");
		return NULL;
	}
	sprintf((char*)path, path_template, IdCat, IdSSCat);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(xmlobject->nodesetval->nodeNr == 0){
		sprintf(ptrSSCategory, "%s", "FIN");
		return ptrSSCategory;
	}
	
	ptrSSCategory = xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar *) "name");
	
	xmlFree(path);
	xmlXPathFreeObject(xmlobject);
	xmlXPathFreeContext(xmlperms_context);
	
	return ptrSSCategory;
}

char *getArticleNameBySousCategory(xmlDocPtr ptrFic, int IdCat, int IdSSCat, int IdArt){
	xmlXPathContextPtr xmlperms_context = NULL;
	xmlXPathObjectPtr xmlobject;
	const char path_template[] = "/liste_articles/category[%d]/sous_category[%d]/article[%d]/name/text()";
	xmlChar *path;
	char *ptrNameCategory;
	
	xmlXPathInit();
	xmlperms_context = xmlXPathNewContext(ptrFic);
	
	path = malloc(sizeof(path_template)+sizeof(int));
	ptrNameCategory = malloc(sizeof(char)*50);
	
	if(!path || !ptrNameCategory){
		fprintf(stderr, "Impossible d'allouer de la memoire\n");
		return NULL;
	}
	sprintf((char*)path, path_template, IdCat, IdSSCat, IdArt);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(!(xmlobject->nodesetval)){
		sprintf(ptrNameCategory, "%s", "FIN");
		return ptrNameCategory;
	}
	
	sprintf(ptrNameCategory, (char *)xmlobject->nodesetval->nodeTab[0]->content);
	
	xmlFree(path);
	xmlXPathFreeObject(xmlobject);
	xmlXPathFreeContext(xmlperms_context);
	
	return ptrNameCategory;
}

char *getArticlePriceBySousCategory(xmlDocPtr ptrFic, int IdCat, int IdSSCat, int IdArt){
	xmlXPathContextPtr xmlperms_context = NULL;
	xmlXPathObjectPtr xmlobject;
	const char path_template[] = "/liste_articles/category[%d]/sous_category[%d]/article[%d]/prix/text()";
	xmlChar *path;
	char *ptrPriceCategory;
	
	xmlXPathInit();
	xmlperms_context = xmlXPathNewContext(ptrFic);
	
	path = malloc(sizeof(path_template)+sizeof(int));
	ptrPriceCategory = malloc(sizeof(char)*50);
	
	if(!path || !ptrPriceCategory){
		fprintf(stderr, "Impossible d'allouer de la memoire\n");
		return NULL;
	}
	sprintf((char*)path, path_template, IdCat, IdSSCat, IdArt);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(!(xmlobject->nodesetval)){
		sprintf(ptrPriceCategory, "%s", "FIN");
		return ptrPriceCategory;
	}
	
	sprintf(ptrPriceCategory, (char *)xmlobject->nodesetval->nodeTab[0]->content);
	
	xmlFree(path);
	xmlXPathFreeObject(xmlobject);
	xmlXPathFreeContext(xmlperms_context);
	
	return ptrPriceCategory;
}
